@@ -41,17 +41,44 @@ import tempfile
 from time import sleep
 from httplib2 import (
     Http,
-    SSLHandshakeError,
+    proxy_info_from_environment,
     urlnorm,
     )
-import simplejson
-from cStringIO import StringIO
+try:
+    from httplib2 import proxy_info_from_environment
+except ImportError:
+    from httplib2 import ProxyInfo
+    proxy_info_from_environment = ProxyInfo.from_environment
+try:
+    # Python 3 and sufficiently modern Python 2 versions.
+    from json import dumps
+except ImportError:
+    from simplejson import dumps
+
+try:
+    # Python 3.
+    from io import BytesIO
+except ImportError:
+    from cStringIO import StringIO as BytesIO
+
+try:
+    # Python 3.
+    from urllib.parse import urlencode
+except ImportError:
+    from urllib import urlencode
 
-from urllib import urlencode
 from wadllib.application import Application
 from lazr.uri import URI
-from errors import error_for, HTTPError
-from _json import DatetimeJSONEncoder
+from lazr.restfulclient.errors import error_for, HTTPError
+from lazr.restfulclient._json import DatetimeJSONEncoder
+
+if bytes is str:
+    # Python 2
+    unicode_type = unicode
+    str_types = basestring
+else:
+    unicode_type = str
+    str_types = str
 
 
 # A drop-in replacement for httplib2's safename.
@@ -64,19 +91,24 @@ def safename(filename):
     """
 
     try:
-        if re_url_scheme.match(filename):
-            if isinstance(filename,str):
+        if isinstance(filename, unicode_type):
+            filename_match = filename.encode('utf-8')
+        else:
+            filename_match = filename
+
+        if re_url_scheme.match(filename_match):
+            if isinstance(filename, bytes):
                 filename = filename.decode('utf-8')
                 filename = filename.encode('idna')
             else:
                 filename = filename.encode('idna')
     except UnicodeError:
         pass
-    if isinstance(filename,unicode):
-        filename=filename.encode('utf-8')
+    if isinstance(filename, unicode_type):
+        filename = filename.encode('utf-8')
     filemd5 = _md5(filename).hexdigest()
-    filename = re_url_scheme.sub("", filename)
-    filename = re_slash.sub(",", filename)
+    filename = re_url_scheme.sub(b"", filename)
+    filename = re_slash.sub(b",", filename)
 
     # This is the part that we changed. In stock httplib2, the
     # filename is trimmed if it's longer than 200 characters, and then
@@ -96,7 +128,7 @@ def safename(filename):
     maximum_length_before_md5_sum = maximum_filename_length - 32 - 1
     if len(filename) > maximum_length_before_md5_sum:
         filename=filename[:maximum_length_before_md5_sum]
-    return ",".join((filename, filemd5))
+    return ",".join((filename.decode('utf-8'), filemd5))
 
 
 def ssl_certificate_validation_disabled():
@@ -127,7 +159,7 @@ class RestfulHttp(Http):
     maximum_cache_filename_length = 143
 
     def __init__(self, authorizer=None, cache=None, timeout=None,
-                 proxy_info=None):
+                 proxy_info=proxy_info_from_environment):
         cert_disabled = ssl_certificate_validation_disabled()
         super(RestfulHttp, self).__init__(
             cache, timeout, proxy_info,
@@ -140,7 +172,7 @@ class RestfulHttp(Http):
     def _request(self, conn, host, absolute_uri, request_uri, method, body,
                  headers, redirections, cachekey):
         """Use the authorizer to authorize an outgoing request."""
-        if headers.has_key('authorization'):
+        if 'authorization' in headers:
             # There's an authorization header left over from a
             # previous request that resulted in a redirect. Resources
             # protected by OAuth or HTTP Digest must send a distinct
@@ -184,7 +216,7 @@ class AtomicFileCache(object):
         self._get_safe_name = safe
         try:
              os.makedirs(self._cache_dir)
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.EEXIST:
                 raise
 
@@ -217,7 +249,7 @@ class AtomicFileCache(object):
                 return f.read()
             finally:
                 f.close()
-        except (IOError, OSError), e:
+        except (IOError, OSError) as e:
             if e.errno != errno.ENOENT:
                 raise
 
@@ -249,7 +281,7 @@ class AtomicFileCache(object):
         cache_full_path = self._get_key_path(key)
         try:
             os.remove(cache_full_path)
-        except OSError, e:
+        except OSError as e:
             if e.errno != errno.ENOENT:
                 raise
 
@@ -286,14 +318,15 @@ class MultipleRepresentationCache(AtomicFileCache):
             key = key + '-' + self.request_media_type
         return safename(key)
 
-
     def _getCachedHeader(self, uri, header):
         """Retrieve a cached value for an HTTP header."""
         (scheme, authority, request_uri, cachekey) = urlnorm(uri)
         cached_value = self.get(cachekey)
         header_start = header + ':'
+        if not isinstance(header_start, bytes):
+            header_start = header_start.encode('utf-8')
         if cached_value is not None:
-            for line in StringIO(cached_value):
+            for line in BytesIO(cached_value):
                 if line.startswith(header_start):
                     return line[len(header_start):].strip()
         return None
@@ -316,7 +349,7 @@ class Browser:
         if cache is None:
             cache = tempfile.mkdtemp()
             atexit.register(shutil.rmtree, cache)
-        if isinstance(cache, basestring):
+        if isinstance(cache, str_types):
             cache = MultipleRepresentationCache(cache)
         self._connection = service_root.httpFactory(
             credentials, cache, timeout, proxy_info)
@@ -348,28 +381,18 @@ class Browser:
                              "don't have the server-side permission to see.")
 
         # Add extra headers for the request.
-        headers = {'Accept' : media_type}
+        headers = {'Accept': media_type}
         if self.user_agent is not None:
             headers['User-Agent'] = self.user_agent
         if isinstance(self._connection.cache, MultipleRepresentationCache):
             self._connection.cache.request_media_type = media_type
         if extra_headers is not None:
             headers.update(extra_headers)
-        # Make the request.
-        try:
-            response, content = self._request_and_retry(
-                str(url), method=method, body=data, headers=headers)
-        except SSLHandshakeError, e:
-            msg = str(e)
-            if "SSL3_GET_SERVER_CERTIFICATE:certificate verify failed" in msg:
-                raise SSLHandshakeError(
-                    "%s: perhaps set LP_DISABLE_SSL_CERTIFICATE_VALIDATION if "
-                    "certificate validation failed and you genuinely trust the "
-                    "remote server." % (e,))
-            raise
+        response, content = self._request_and_retry(
+            str(url), method=method, body=data, headers=headers)
         if response.status == 304:
             # The resource didn't change.
-            if content == '':
+            if content == b'':
                 if ('If-None-Match' in headers
                     or 'If-Modified-Since' in headers):
                     # The caller made a conditional request, and the
@@ -410,7 +433,7 @@ class Browser:
 
     def get(self, resource_or_uri, headers=None, return_response=False):
         """GET a representation of the given resource or URI."""
-        if isinstance(resource_or_uri, (basestring, URI)):
+        if isinstance(resource_or_uri, (str_types, URI)):
             url = resource_or_uri
         else:
             method = resource_or_uri.get_method('get')
@@ -424,7 +447,10 @@ class Browser:
         """GET a WADL representation of the resource at the requested url."""
         wadl_type = 'application/vnd.sun.wadl+xml'
         response, content = self._request(url, media_type=wadl_type)
-        return Application(str(url), content)
+        url = str(url)
+        if not isinstance(content, bytes):
+            content = content.encode('utf-8')
+        return Application(url, content)
 
     def post(self, url, method_name, **kws):
         """POST a request to the web service."""
@@ -459,5 +485,5 @@ class Browser:
             headers['If-Match'] = cached_etag
 
         return self._request(
-            url, simplejson.dumps(representation, cls=DatetimeJSONEncoder),
+            url, dumps(representation, cls=DatetimeJSONEncoder),
             'PATCH', extra_headers=extra_headers)
