@@ -19,7 +19,11 @@
 """OAuth classes for use with lazr.restfulclient."""
 
 
-from ConfigParser import SafeConfigParser
+try:
+    # Python 3, SafeConfigParser was renamed to just ConfigParser.
+    from configparser import ConfigParser as SafeConfigParser
+except ImportError:
+    from ConfigParser import SafeConfigParser
 import os
 import platform
 import stat
@@ -105,7 +109,7 @@ class SystemWideConsumer(Consumer):
         """
         try:
             distname, version, release_id = platform.linux_distribution()
-        except Exception, e:
+        except Exception:
             # This can happen in pre-2.6 versions of Python. We don't
             # use platform.dist() because it looks like old versions
             # of dist can give inconsistent answers on a single
@@ -155,7 +159,8 @@ class OAuthAuthorizer(HttpAuthorizer):
         """
         # Attempt to load the access token from the file.
         parser = SafeConfigParser()
-        parser.readfp(readable_file)
+        reader = getattr(parser, 'read_file', parser.readfp)
+        reader(readable_file)
         # Check the version number and extract the access token and
         # secret.  Then convert these to the appropriate instances.
         if not parser.has_section(CREDENTIALS_FILE_VERSION):
