@@ -27,16 +27,12 @@ import doctest
 import os
 from pkg_resources import (
     resource_filename, resource_exists, resource_listdir, cleanup_resources)
-import unittest
 import wsgi_intercept
 from wsgi_intercept.httplib2_intercept import install, uninstall
 
-from zope.component import getUtility
-
-from lazr.restful.example.base.interfaces import IFileManager
-from lazr.restful.example.base.tests.test_integration import WSGILayer
-from lazr.restful.testing.webservice import WebServiceApplication
-
+# We avoid importing anything from lazr.restful into the module level,
+# so that standalone_tests() can run without any support from
+# lazr.restful.
 
 DOCTEST_FLAGS = (
     doctest.ELLIPSIS |
@@ -45,31 +41,51 @@ DOCTEST_FLAGS = (
 
 
 def setUp(test):
+    from lazr.restful.example.base.tests.test_integration import WSGILayer
     install()
     wsgi_intercept.add_wsgi_intercept(
         'cookbooks.dev', 80, WSGILayer.make_application)
 
 
 def tearDown(test):
+    from lazr.restful.example.base.interfaces import IFileManager
+    from zope.component import getUtility
     uninstall()
     file_manager = getUtility(IFileManager)
     file_manager.files = {}
     file_manager.counter = 0
 
 
-def additional_tests():
-    "Run the doc tests (README.txt and docs/*, if any exist)"
+def find_doctests(suffix):
+    """Find doctests matching a certain suffix."""
+    # Always include README.txt.
     doctest_files = [
         os.path.abspath(resource_filename('lazr.restfulclient', 'README.txt'))]
+    # Match other doctests against the suffix.
     if resource_exists('lazr.restfulclient', 'docs'):
         for name in resource_listdir('lazr.restfulclient', 'docs'):
-            if name.endswith('.txt'):
+            if name.endswith(suffix):
                 doctest_files.append(
                     os.path.abspath(
-                        resource_filename('lazr.restfulclient', 'docs/%s' % name)))
+                        resource_filename(
+                            'lazr.restfulclient', 'docs/%s' % name)))
+    return doctest_files
+
+
+def additional_tests():
+    "Run the doc tests (README.txt and docs/*, if any exist)"
+    from lazr.restful.example.base.tests.test_integration import WSGILayer
     kwargs = dict(module_relative=False, optionflags=DOCTEST_FLAGS,
                   setUp=setUp, tearDown=tearDown)
     atexit.register(cleanup_resources)
-    suite = doctest.DocFileSuite(*doctest_files, **kwargs)
+    suite = doctest.DocFileSuite(*find_doctests('.txt'), **kwargs)
     suite.layer = WSGILayer
     return suite
+
+
+def standalone_tests():
+    "Run the tests that don't need lazr.restful."
+    kwargs = dict(module_relative=False, optionflags=DOCTEST_FLAGS)
+    atexit.register(cleanup_resources)
+    suite = doctest.DocFileSuite(*find_doctests('.standalone.txt'), **kwargs)
+    return suite
